@@ -2,7 +2,7 @@
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/charts/linecharts.py
 
-__version__=''' $Id: linecharts.py 3345 2008-12-12 17:55:22Z damian $ '''
+__version__=''' $Id: linecharts.py 3604 2009-11-27 16:35:29Z meitham $ '''
 __doc__="""This modules defines a very preliminary Line Chart example."""
 
 from reportlab.lib import colors
@@ -10,7 +10,6 @@ from reportlab.lib.validators import isNumber, isColor, isColorOrNone, isListOfS
                                     isListOfStringsOrNone, SequenceOf, isBoolean, NoneOr, \
                                     isListOfNumbersOrNone, isStringOrNone
 from reportlab.lib.attrmap import *
-from reportlab.lib.formatters import Formatter
 from reportlab.graphics.widgetbase import Widget, TypedPropertyCollection, PropHolder
 from reportlab.graphics.shapes import Line, Rect, Group, Drawing, Polygon, PolyLine
 from reportlab.graphics.widgets.signsandsymbols import NoEntry
@@ -25,9 +24,9 @@ class LineChartProperties(PropHolder):
         strokeWidth = AttrMapValue(isNumber, desc='Width of a line.'),
         strokeColor = AttrMapValue(isColorOrNone, desc='Color of a line.'),
         strokeDashArray = AttrMapValue(isListOfNumbersOrNone, desc='Dash array of a line.'),
-        symbol = AttrMapValue(NoneOr(isSymbol), desc='Widget placed at data points.'),
-        shader = AttrMapValue(None, desc='Shader Class.'),
-        filler = AttrMapValue(None, desc='Filler Class.'),
+        symbol = AttrMapValue(NoneOr(isSymbol), desc='Widget placed at data points.',advancedUsage=1),
+        shader = AttrMapValue(None, desc='Shader Class.',advancedUsage=1),
+        filler = AttrMapValue(None, desc='Filler Class.',advancedUsage=1),
         name = AttrMapValue(isStringOrNone, desc='Name of the line.'),
         )
 
@@ -109,8 +108,8 @@ class HorizontalLineChart(LineChart):
     """
 
     _attrMap = AttrMap(BASE=LineChart,
-        useAbsolute = AttrMapValue(isNumber, desc='Flag to use absolute spacing values.'),
-        lineLabelNudge = AttrMapValue(isNumber, desc='Distance between a data point and its label.'),
+        useAbsolute = AttrMapValue(isNumber, desc='Flag to use absolute spacing values.',advancedUsage=1),
+        lineLabelNudge = AttrMapValue(isNumber, desc='Distance between a data point and its label.',advancedUsage=1),
         lineLabels = AttrMapValue(None, desc='Handle to the list of data point labels.'),
         lineLabelFormat = AttrMapValue(None, desc='Formatting string or function used for data point labels.'),
         lineLabelArray = AttrMapValue(None, desc='explicit array of line label values, must match size of data if present.'),
@@ -121,9 +120,9 @@ class HorizontalLineChart(LineChart):
         categoryAxis = AttrMapValue(None, desc='Handle of the category axis.'),
         categoryNames = AttrMapValue(isListOfStringsOrNone, desc='List of category names.'),
         data = AttrMapValue(None, desc='Data to be plotted, list of (lists of) numbers.'),
-        inFill = AttrMapValue(isBoolean, desc='Whether infilling should be done.'),
-        reversePlotOrder = AttrMapValue(isBoolean, desc='If true reverse plot order.'),
-        annotations = AttrMapValue(None, desc='list of callables, will be called with self, xscale, yscale.'),
+        inFill = AttrMapValue(isBoolean, desc='Whether infilling should be done.',advancedUsage=1),
+        reversePlotOrder = AttrMapValue(isBoolean, desc='If true reverse plot order.',advancedUsage=1),
+        annotations = AttrMapValue(None, desc='list of callables, will be called with self, xscale, yscale.',advancedUsage=1),
         )
 
     def __init__(self):
@@ -248,16 +247,14 @@ class HorizontalLineChart(LineChart):
                     labelText = None
             else:
                 labelText = labelFmt % labelValue
-        elif isinstance(labelFmt, Formatter):
-            labelText = labelFmt(labelValue)
         elif callable(labelFmt):
             labelText = labelFmt(labelValue)
         else:
-            msg = "Unknown formatter type %s, expected string or function"
-            raise Exception, msg % labelFmt
+            raise ValueError("Unknown formatter type %s, expected string or function"%labelFmt)
 
         if labelText:
             label = self.lineLabels[(rowNo, colNo)]
+            if not label.visible: return
             # Make sure labels are some distance off the data point.
             if y > 0:
                 label.setOrigin(x, y + self.lineLabelNudge)
@@ -368,9 +365,13 @@ class HorizontalLineChart(LineChart):
 
         g.add(cA)
         g.add(vA)
-        cA.makeGrid(g,parent=self,dim=vA.getGridDims)
-        vA.makeGrid(g,parent=self,dim=cA.getGridDims)
+        cAdgl = getattr(cA,'drawGridLast',False)
+        vAdgl = getattr(vA,'drawGridLast',False)
+        if not cAdgl: cA.makeGrid(g,parent=self,dim=vA.getGridDims)
+        if not vAdgl: vA.makeGrid(g,parent=self,dim=cA.getGridDims)
         g.add(self.makeLines())
+        if cAdgl: cA.makeGrid(g,parent=self,dim=vA.getGridDims)
+        if vAdgl: vA.makeGrid(g,parent=self,dim=cA.getGridDims)
         for a in getattr(self,'annotations',()): g.add(a(self,cA.scale,vA.scale))
         return g
 
