@@ -1,24 +1,19 @@
 #Copyright ReportLab Europe Ltd. 2000-2006
 #see license.txt for license details
 # $URI:$
-__version__=''' $Id: utils.py 3397 2009-01-23 16:31:04Z rgbecker $ '''
+__version__=''' $Id: utils.py 3624 2009-12-18 10:52:19Z rgbecker $ '''
 __doc__='''Gazillions of miscellaneous internal utility functions'''
 
-import string, os, sys, imp, time
+import os, sys, imp, time
 try:
     from hashlib import md5
 except:
     from md5 import md5
 from reportlab.lib.logger import warnOnce
-from types import *
 from rltempfile import get_rl_tempfile, get_rl_tempdir, _rl_getuid
-SeqTypes = (ListType,TupleType)
-if sys.hexversion<0x2020000:
-    def isSeqType(v):
-        return type(v) in SeqTypes
-else:
-    def isSeqType(v):
-        return isinstance(v,(tuple,list))
+
+def isSeqType(v,_st=(tuple,list)):
+    return isinstance(v,_st)
 
 if sys.hexversion<0x2030000:
     True = 1
@@ -131,7 +126,7 @@ if os.name == 'mac':
         try:
             if creatorcode is None or filetype is None and ext is not None:
                 try:
-                    creatorcode, filetype = _KNOWN_MAC_EXT[string.upper(ext)]
+                    creatorcode, filetype = _KNOWN_MAC_EXT[ext.upper()]
                 except:
                     return
             macfs.FSSpec(filename).SetCreatorType(creatorcode,filetype)
@@ -234,6 +229,7 @@ except ImportError:
     _tz_re = re.compile('0+$')
     del re
     def fp_str(*a):
+        '''convert separate arguments (or single sequence arg) into space separated numeric strings'''
         if len(a)==1 and isSeqType(a[0]): a = a[0]
         s = []
         A = s.append
@@ -251,13 +247,13 @@ except ImportError:
                         print i, n
                         raise
                 A((n[0]!='0' or len(n)==1) and n or n[1:])
-        return string.join(s)
+        return ' '.join(s)
 
 #hack test for comma users
 if ',' in fp_str(0.25):
     _FP_STR = fp_str
     def fp_str(*a):
-        return string.replace(apply(_FP_STR,a),',','.')
+        return _FP_STR(*a).replace(',','.')
 
 def recursiveImport(modulename, baseDir=None, noCWD=0, debug=0):
     """Dynamically imports possible packagized module, or raises ImportError"""
@@ -307,11 +303,11 @@ def recursiveGetAttr(obj, name):
 def recursiveSetAttr(obj, name, value):
     "Can call down into e.g. object1.object2[4].attr = value"
     #get the thing above last.
-    tokens = string.split(name, '.')
+    tokens = name.split('.')
     if len(tokens) == 1:
         setattr(obj, name, value)
     else:
-        most = string.join(tokens[:-1], '.')
+        most = '.'.join(tokens[:-1])
         last = tokens[-1]
         parent = recursiveGetAttr(obj, most)
         setattr(parent, last, value)
@@ -345,7 +341,6 @@ else:
         except ImportError:
             Image = None
     haveImages = Image is not None
-    if haveImages: del Image
 
 try:
     from cStringIO import StringIO as __StringIO
@@ -371,19 +366,19 @@ def getArgvDict(**kw):
         if func:
             v = func(av)
         else:
-            t = type(v)
-            if t is StringType:
+            if isinstance(v,basestring):
+                if isinstance(v,unicode): v = v.encode('utf8')
                 v = av
-            elif t is FloatType:
+            elif isinstance(v,float):
                 v = float(av)
-            elif t is IntType:
+            elif isinstance(v,int):
                 v = int(av)
-            elif t is ListType:
+            elif isinstance(v,list):
                 v = list(eval(av))
-            elif t is TupleType:
+            elif isinstance(v,tuple):
                 v = tuple(eval(av))
             else:
-                raise TypeError, "Can't convert string '%s' to %s" % (av,str(t))
+                raise TypeError("Can't convert string %r to %s" % (av,type(v)))
         return v
 
     A = sys.argv[1:]
@@ -396,7 +391,7 @@ def getArgvDict(**kw):
         handled = 0
         ke = k+'='
         for a in A:
-            if string.find(a,ke)==0:
+            if a.find(ke)==0:
                 av = a[len(ke):]
                 A.remove(a)
                 R[k] = handleValue(v,av,func)
@@ -420,7 +415,7 @@ def _className(self):
     '''Return a shortened class name'''
     try:
         name = self.__class__.__name__
-        i=string.rfind(name,'.')
+        i=name.rfind('.')
         if i>=0: return name[i+1:]
         return name
     except AttributeError:
@@ -523,8 +518,7 @@ def rl_get_module(name,dir):
 
 def _isPILImage(im):
     try:
-        from PIL.Image import Image
-        return isinstance(im,Image)
+        return isinstance(im,Image.Image)
     except ImportError:
         return 0
 
@@ -602,8 +596,7 @@ class ImageReader(object):
             from javax.imageio import ImageIO
             return ImageIO.read(fp)
         else:
-            import PIL.Image
-            return PIL.Image.open(fp)
+            return Image.open(fp)
 
     def _jpeg_fh(self):
         fp = self.fp
@@ -648,6 +641,7 @@ class ImageReader(object):
                 im = self._image
                 mode = self.mode = im.mode
                 if mode=='RGBA':
+                    if Image.VERSION.startswith('1.1.7'): im.load()
                     self._dataA = ImageReader(im.split()[3])
                     im = im.convert('RGB')
                     self.mode = 'RGB'
@@ -721,7 +715,6 @@ class DebugMemo:
     def __init__(self,fn='rl_dbgmemo.dbg',mode='w',getScript=1,modules=(),capture_traceback=1, stdout=None, **kw):
         import time, socket
         self.fn = fn
-        if mode!='w': return
         if not stdout: 
             self.stdout = sys.stdout
         else:
@@ -729,6 +722,7 @@ class DebugMemo:
                 self.stdout = stdout
             else:
                 self.stdout = open(stdout,'w')
+        if mode!='w': return
         self.store = store = {}
         if capture_traceback and sys.exc_info() != (None,None,None):
             import traceback
@@ -737,7 +731,20 @@ class DebugMemo:
             store['__traceback'] = s.getvalue()
         cwd=os.getcwd()
         lcwd = os.listdir(cwd)
+        pcwd = os.path.dirname(cwd)
+        lpcwd = pcwd and os.listdir(pcwd) or '???'
         exed = os.path.abspath(os.path.dirname(sys.argv[0]))
+        project_version='???'
+        md=None
+        try:
+            import marshal
+            md=marshal.loads(__loader__.get_data('meta_data.mar'))
+            project_version=md['project_version']
+        except:
+            pass
+        env = os.environ
+        K=env.keys()
+        K.sort()
         store.update({  'gmt': time.asctime(time.gmtime(time.time())),
                         'platform': sys.platform,
                         'version': sys.version,
@@ -750,13 +757,17 @@ class DebugMemo:
                         'cwd': cwd,
                         'hostname': socket.gethostname(),
                         'lcwd': lcwd,
+                        'lpcwd': lpcwd,
                         'byteorder': sys.byteorder,
                         'maxint': sys.maxint,
                         'maxint': getattr(sys,'maxunicode','????'),
                         'api_version': getattr(sys,'api_version','????'),
                         'version_info': getattr(sys,'version_info','????'),
                         'winver': getattr(sys,'winver','????'),
-                        'environment': os.environ,
+                        'environment': '\n\t\t\t'.join(['']+['%s=%r' % (k,env[k]) for k in K]),
+                        '__loader__': repr(__loader__),
+                        'project_meta_data': md,
+                        'project_version': project_version,
                         })
         for M,A in (
                 (sys,('getwindowsversion','getfilesystemencoding')),
@@ -785,8 +796,10 @@ class DebugMemo:
         module_versions = {}
         for n,m in sys.modules.items():
             if n=='reportlab' or n=='rlextra' or n[:10]=='reportlab.' or n[:8]=='rlextra.':
-                v = getattr(m,'__version__',None)
-                if v: module_versions[n] = v
+                v = [getattr(m,x,None) for x in ('__version__','__path__','__file__')]
+                if filter(None,v):
+                    v = [v[0]] + filter(None,v[1:])
+                    module_versions[n] = tuple(v)
         store['__module_versions'] = module_versions
         self.store['__payload'] = {}
         self._add(kw)
@@ -801,7 +814,19 @@ class DebugMemo:
 
     def _dump(self,f):
         import pickle
-        pickle.dump(self.store,f)
+        try:
+            pos=f.tell()
+            pickle.dump(self.store,f)
+        except:
+            S=self.store.copy()
+            ff=getStringIO()
+            for k,v in S.iteritems():
+                try:
+                    pickle.dump({k:v},ff)
+                except:
+                    S[k] = '<unpicklable object %r>' % v
+            f.seek(pos,0)
+            pickle.dump(S,f)
 
     def dump(self):
         f = open(self.fn,'wb')
@@ -834,10 +859,11 @@ class DebugMemo:
         K = v.keys()
         K.sort()
         for k in K:
-            vk = v[k]
+            vk = vk0 = v[k]
+            if isinstance(vk,tuple): vk0 = vk[0]
             try:
                 m = recursiveImport(k,sys.path[:],1)
-                d = getattr(m,'__version__',None)==vk and 'SAME' or 'DIFFERENT'
+                d = getattr(m,'__version__',None)==vk0 and 'SAME' or 'DIFFERENT'
             except:
                 m = None
                 d = '??????unknown??????'
@@ -868,6 +894,19 @@ class DebugMemo:
             pprint.pprint(v,self.stdout)
             self._finish(k)
 
+    def _show_extensions(self):
+        for mn in ('_rl_accel','_renderPM','sgmlop','pyRXP','pyRXPU','_imaging','Image'):
+            try:
+                A = [mn].append
+                m = recursiveImport(mn,sys.path[:],1)
+                A(m.__file__)
+                for vn in ('__version__','VERSION','_version','version'):
+                    if hasattr(m,vn):
+                        A('%s=%r' % (vn,getattr(m,vn)))
+            except:
+                A('not found')
+            self._writeln(' '+' '.join(A.__self__))
+
     specials = {'__module_versions': _show_module_versions,
                 '__payload': _show_payload,
                 '__traceback': _show_lines,
@@ -879,7 +918,8 @@ class DebugMemo:
         for k in K:
             if k not in self.specials.keys(): self._writeln('%-15s = %s' % (k,self.store[k]))
         for k in K:
-            if k in self.specials.keys(): apply(self.specials[k],(self,k,self.store[k]))
+            if k in self.specials.keys(): self.specials[k](self,k,self.store[k])
+        self._show_extensions()
 
     def payload(self,name):
         return self.store['__payload'][name]
@@ -941,10 +981,10 @@ def _simpleSplit(txt,mW,SW):
             O.append(t)
             w = w + ws + lt
         else:
-            L.append(string.join(O,' '))
+            L.append(' '.join(O))
             O = [t]
             w = lt
-    if O!=[]: L.append(string.join(O,' '))
+    if O!=[]: L.append(' '.join(O))
     return L
 
 def simpleSplit(text,fontName,fontSize,maxWidth):
@@ -1006,3 +1046,86 @@ def prev_this_next(items):
     except StopIteration:
         pass
     return itertools.izip(prev, this, next)
+
+def commasplit(s):
+    '''
+    Splits the string s at every unescaped comma and returns the result as a list.
+    To escape a comma, double it. Individual items are stripped.
+    To avoid the ambiguity of 3 successive commas to denote a comma at the beginning
+    or end of an item, add a space between the item seperator and the escaped comma.
+    
+    >>> commasplit('a,b,c')
+    ['a', 'b', 'c']
+    >>> commasplit('a,, , b , c    ')
+    ['a,', 'b', 'c']
+    >>> commasplit('a, ,,b, c')
+    ['a', ',b', 'c']
+    '''
+    n = len(s)-1
+    s += ' '
+    i = 0
+    r=['']
+    while i<=n:
+        if s[i]==',':
+            if s[i+1]==',':
+                r[-1]+=','
+                i += 1
+            else:
+                r[-1] = r[-1].strip()
+                if i!=n: r.append('')
+        else:
+            r[-1] += s[i]
+        i+=1
+    r[-1] = r[-1].strip()
+    return r
+    
+def commajoin(l):
+    '''
+    Inverse of commasplit, except that whitespace around items is not conserved.
+    Adds more whitespace than needed for simplicity and performance.
+    
+    >>> commasplit(commajoin(['a', 'b', 'c']))
+    ['a', 'b', 'c']
+    >>> commasplit((commajoin(['a,', ' b ', 'c']))
+    ['a,', 'b', 'c']
+    >>> commasplit((commajoin(['a ', ',b', 'c']))
+    ['a', ',b', 'c']    
+    '''
+    return ','.join([ ' ' + i.replace(',', ',,') + ' ' for i in l ])
+
+def findInPaths(fn,paths,isfile=True,fail=False):
+    '''search for relative files in likely places'''
+    exists = isfile and os.path.isfile or os.path.isdir
+    if exists(fn): return fn
+    pjoin = os.path.join
+    if not os.path.isabs(fn):
+        for p in paths:
+            pfn = pjoin(p,fn)
+            if exists(pfn):
+                return pfn
+    if fail: raise ValueError('cannot locate %r with paths=%r' % (fn,paths))
+    return fn
+
+def annotateException(msg,enc='utf8'):
+    '''add msg to the args of an existing exception'''
+    t,v,b=sys.exc_info()
+    e = -1
+    A = list(v.args)
+    for i,a in enumerate(A):
+        if isinstance(a,basestring):
+            e = i
+            break
+    if e>=0:
+        if isinstance(a,unicode):
+            if not isinstance(msg,unicode):
+                msg=msg.decode(enc)
+        else:
+            if isinstance(msg,unicode):
+                msg=msg.encode(enc)
+            else:
+                msg = str(msg)
+        A[e] += msg
+    else:
+        A.append(msg)
+    v.args = tuple(A)
+    raise t,v,b
